from cpython.bytes cimport (PyBytes_GET_SIZE,
                            PyBytes_AS_STRING,
                            PyBytes_Size,
                            PyBytes_FromString,
                            PyBytes_FromStringAndSize)
from cpython.float cimport PyFloat_FromDouble
from cpython.long cimport PyLong_FromLong

from cpython.ref cimport (Py_INCREF, Py_DECREF, PyTypeObject)

from libc.stdio cimport (FILE, stdout)
from libc.stdio cimport stdout
from libc.stdlib cimport malloc, calloc, free
from libc.string cimport memcpy
from libc.stdint cimport (uint8_t, int8_t,
                          uint16_t, int16_t,
                          uint32_t, int32_t,
                          uint64_t, int64_t,
                          uintptr_t)
from libc.stddef cimport ptrdiff_t

from libc cimport limits
from libcpp.vector cimport vector

cdef extern from "Python.h":
    object PyUnicode_FromStringAndSize(const char *u, Py_ssize_t size)
    object PyUnicode_FromString(const char *u)

# Numpy imports
"""
cdef extern from "numpyFlags.h":
    # Include 'numpyFlags.h' into the generated C code to disable warning.
    # This must be included before numpy is cimported
    pass
"""

import numpy as np
cimport numpy as np

cdef extern from "numpy/arrayobject.h":
    # Steals a reference to dtype, need to incref the dtype
    object PyArray_NewFromDescr(PyTypeObject* subtype,
                                np.dtype descr,
                                int nd,
                                np.npy_intp* dims,
                                np.npy_intp* strides,
                                void* data,
                                int flags,
                                object obj)
    # Steals a reference to dtype, need to incref the dtype
    object PyArray_Scalar(void* ptr, np.dtype descr, object itemsize)
    void PyArray_ENABLEFLAGS(np.ndarray arr, int flags)
    void* PyDataMem_NEW(size_t nbytes)
    void* PyDataMem_RENEW(void* data, size_t nbytes)
    void PyDataMem_FREE(void* data)



from .libtiledb import DenseArrayImpl, SparseArrayImpl

# Extensible (pure Python) array class definitions inheriting from the
# Cython implementation.

# NOTE: the mixin import must be inside the __new__ initializer because it
#       needs to be deferred. tiledb.cloud is not yet known to the importer
#       when this code is imported.
#       TODO: might be possible to work-around/simplify by using
#       import meta-hooks instead.


class DenseArray(DenseArrayImpl):
    """Class representing a dense TileDB array.

    Inherits properties and methods of :py:class:`tiledb.Array` and
    implements `__setitem__` and `__getitem__` for dense array indexing
    and assignment.
    """

    def __new__(cls, *args, **kwargs):
        try:
            from tiledb.cloud import cloudarray

            cls.__bases__ += (cloudarray.CloudArray,)
        except (ImportError, TypeError):
            # If we couldn't import, tiledb.cloud isn't installed
            # If there was a TypeError, CloudArray was already added (by another thread)
            pass
        try:
            # Mixin initialization completed: delete this method for subsequent calls
            del DenseArray.__new__
        except AttributeError:
            # This method is already deleted (by another thread)
            pass
        return super().__new__(cls, *args, **kwargs)


class SparseArray(SparseArrayImpl):
    """Class representing a sparse TileDB array.

    Inherits properties and methods of :py:class:`tiledb.Array` and
    implements `__setitem__` and `__getitem__` for sparse array indexing
    and assignment.
    """

    def __new__(cls, *args, **kwargs):
        try:
            from tiledb.cloud import cloudarray

            cls.__bases__ += (cloudarray.CloudArray,)
        except (ImportError, TypeError):
            # If we couldn't import, tiledb.cloud isn't installed
            # If there was a TypeError, CloudArray was already added (by another thread)
            pass
        try:
            # Mixin initialization completed: delete this method for subsequent calls
            del SparseArray.__new__
        except AttributeError:
            # This method is already deleted (by another thread)
            pass
        return super().__new__(cls, *args, **kwargs)



import urllib

# from tiledb.tests.common import paths_equal


def paths_equal(path1, path2):
    p1 = urllib.parse.urlparse(path1)
    p2 = urllib.parse.urlparse(path2)

    if p1.scheme == p2.scheme:
        pass
    elif "file" in (p1.scheme, p2.scheme):
        # if
        scheme_eq = p1.scheme == "file" or p1.scheme == ""
        scheme_eq |= p2.scheme == "file" or p2.scheme == ""
        return p1.path == p2.path and scheme_eq

    return p1.schema == p2.schema and p1.path == p2.path and p1.netloc == p2.netloc



import numpy as np
import tiledb
import hypothesis
import time
import tempfile
import os

from tiledb import cc as lt
from tiledb.tests.common import paths_equal

import pytest


def test_dir(tmp_path):
    ctx = lt.Context()
    vfs = lt.VFS(ctx)

    path = os.path.join(tmp_path, "test_dir")

    vfs.create_dir(path)
    assert vfs.is_dir(path) == True
    assert vfs.dir_size(path) == 0
    vfs.remove_dir(path)
    assert vfs.is_dir(path) == False


def test_file_handle(tmp_path):
    ctx = lt.Context()
    vfs = lt.VFS(ctx)

    path = os.path.join(tmp_path, "test_file_handle")

    fh = lt.FileHandle(ctx, vfs, path, lt.VFSMode.WRITE)
    fh.write(b"Hello")

    fh = lt.FileHandle(ctx, vfs, path, lt.VFSMode.READ)
    assert fh.read(0, 5) == b"Hello"

    fh = lt.FileHandle(ctx, vfs, path, lt.VFSMode.APPEND)
    fh.write(b", world!")

    fh = lt.FileHandle(ctx, vfs, path, lt.VFSMode.READ)
    assert fh.read(0, 13) == b"Hello, world!"

    assert fh.closed == False
